@@ -35,10 +35,14 @@ Existing unit tests that use the real file system can be retrofitted to use
 pyfakefs by simply changing their base class from `:py:class`unittest.TestCase`
 to `:py:class`pyfakefs.fake_filesystem_unittest.TestCase`.
 """
+import _io  # type:ignore [import]
 import doctest
 import functools
+import genericpath
 import inspect
+import io
 import linecache
+import os
 import shutil
 import sys
 import tempfile
@@ -309,7 +313,7 @@ class TestCase(unittest.TestCase, TestCaseMixin):
         path in the fake file system.
 
         **This method is deprecated** in favor of
-        :py:meth:`FakeFilesystem..add_real_file`.
+        :py:meth:`FakeFilesystem.add_real_file`.
         `copyRealFile()` is retained with limited functionality for backward
         compatibility only.
 
@@ -366,8 +370,6 @@ class Patcher:
     '''Stub nothing that is imported within these modules.
     `sys` is included to prevent `sys.path` from being stubbed with the fake
     `os.path`.
-    The `pytest` and `py` modules are used by pytest and have to access the
-    real file system.
     The `linecache` module is used to read the test file in case of test
     failure to get traceback information before test tear down.
     In order to make sure that reading the test file is not faked,
@@ -376,8 +378,21 @@ class Patcher:
     '''
     SKIPMODULES = {
         None, fake_filesystem, fake_filesystem_shutil,
-        sys, linecache, tokenize
+        sys, linecache, tokenize, os, io, _io, genericpath, os.path
     }
+    if sys.platform == 'win32':
+        import nt  # type:ignore [import]
+        import ntpath
+        SKIPMODULES.add(nt)
+        SKIPMODULES.add(ntpath)
+    else:
+        import posix
+        import posixpath
+        import fcntl
+        SKIPMODULES.add(posix)
+        SKIPMODULES.add(posixpath)
+        SKIPMODULES.add(fcntl)
+
     # caches all modules that do not have file system modules or function
     # to speed up _find_modules
     CACHED_MODULES: Set[ModuleType] = set()
@@ -391,13 +406,19 @@ class Patcher:
 
     IS_WINDOWS = sys.platform in ('win32', 'cygwin')
 
-    SKIPNAMES = {'os', 'path', 'io', 'genericpath', 'fcntl',
-                 OS_MODULE, PATH_MODULE}
+    SKIPNAMES: Set[str] = set()
 
     # hold values from last call - if changed, the cache has to be invalidated
     PATCHED_MODULE_NAMES: Set[str] = set()
     ADDITIONAL_SKIP_NAMES: Set[str] = set()
     PATCH_DEFAULT_ARGS = False
+    PATCHER = None
+    REF_COUNT = 0
+
+    def __new__(cls, *args, **kwargs):
+        if cls.PATCHER is None:
+            cls.PATCHER = super().__new__(cls)
+        return cls.PATCHER
 
     def __init__(self, additional_skip_names: Optional[
         List[Union[str, ModuleType]]] = None,
@@ -439,7 +460,8 @@ class Patcher:
                 feature, this argument allows to turn it off in case it
                 causes any problems.
         """
-
+        if self.REF_COUNT > 0:
+            return
         if not allow_root_user:
             # set non-root IDs even if the real user is root
             set_uid(1)
@@ -605,34 +627,26 @@ class Patcher:
                       name: str,
                       module_names: List[str]) -> bool:
         try:
-            # check for __name__ first and ignore the AttributeException
-            # if it does not exist - avoids calling expansive ismodule
-            if mod.__name__ in module_names and inspect.ismodule(mod):
-                return True
+            return (inspect.ismodule(mod) and
+                    mod.__name__ in module_names
+                    or inspect.isclass(mod) and
+                    mod.__module__ in self._class_modules.get(name, []))
         except Exception:
-            pass
-        try:
-            if (name in self._class_modules and
-                    mod.__module__ in self._class_modules[name]):
-                return inspect.isclass(mod)
-        except Exception:
-            # handle AttributeError and any other exception possibly triggered
-            # by side effects of inspect methods
-            pass
-        return False
+            # handle cases where the module has no __name__ or __module__
+            # attribute - see #460, and any other exception triggered
+            # by inspect functions
+            return False
 
     def _is_fs_function(self, fct: FunctionType) -> bool:
         try:
-            # check for __name__ first and ignore the AttributeException
-            # if it does not exist - avoids calling expansive inspect
-            # methods in most cases
-            return (fct.__name__ in self._fake_module_functions and
+            return ((inspect.isfunction(fct) or
+                     inspect.isbuiltin(fct)) and
+                    fct.__name__ in self._fake_module_functions and
                     fct.__module__ in self._fake_module_functions[
-                        fct.__name__] and
-                    (inspect.isfunction(fct) or inspect.isbuiltin(fct)))
+                        fct.__name__])
         except Exception:
-            # handle AttributeError and any other exception possibly triggered
-            # by side effects of inspect methods
+            # handle cases where the function has no __name__ or __module__
+            # attribute, or any other exception in inspect functions
             return False
 
     def _def_values(
@@ -745,6 +759,9 @@ class Patcher:
         """Bind the file-related modules to the :py:mod:`pyfakefs` fake
         modules real ones.  Also bind the fake `file()` and `open()` functions.
         """
+        self.__class__.REF_COUNT += 1
+        if self.__class__.REF_COUNT > 1:
+            return
         self.has_fcopy_file = (sys.platform == 'darwin' and
                                hasattr(shutil, '_HAS_FCOPYFILE') and
                                shutil._HAS_FCOPYFILE)
@@ -834,11 +851,15 @@ class Patcher:
 
     def tearDown(self, doctester: Any = None):
         """Clear the fake filesystem bindings created by `setUp()`."""
+        self.__class__.REF_COUNT -= 1
+        if self.__class__.REF_COUNT > 0:
+            return
         self.stop_patching()
         if self.has_fcopy_file:
             shutil._HAS_FCOPYFILE = True  # type: ignore[attr-defined]
 
         reset_ids()
+        self.__class__.PATCHER = None
 
     def stop_patching(self) -> None:
         if self._patching:
