@@ -8,7 +8,6 @@ def my_fakefs_test(fs):
     fs.create_file('/var/data/xx1.txt')
     assert os.path.exists('/var/data/xx1.txt')
 """
-import _pytest
 import py
 import pytest
 
@@ -16,8 +15,6 @@ from pyfakefs.fake_filesystem_unittest import Patcher
 
 Patcher.SKIPMODULES.add(py)
 Patcher.SKIPMODULES.add(pytest)
-if hasattr(_pytest, "pathlib"):
-    Patcher.SKIPMODULES.add(_pytest.pathlib)
 
 
 @pytest.fixture
@@ -31,3 +28,27 @@ def fs(request):
     patcher.setUp()
     yield patcher.fs
     patcher.tearDown()
+
+
+@pytest.fixture(scope="module")
+def fs_module(request):
+    """ Module-scoped fake filesystem fixture. """
+    if hasattr(request, 'param'):
+        patcher = Patcher(*request.param)
+    else:
+        patcher = Patcher()
+    patcher.setUp()
+    yield patcher.fs
+    patcher.tearDown()
+
+
+@pytest.fixture(scope="session")
+def fs_session(request):
+    """ Session-scoped fake filesystem fixture. """
+    if hasattr(request, 'param'):
+        patcher = Patcher(*request.param)
+    else:
+        patcher = Patcher()
+    patcher.setUp()
+    yield patcher.fs
+    patcher.tearDown()
